// ======================================================================
// cNNTP.cpp - a simple NNTP client
// 
// 122502: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 072102: Benjamin Han <benhdj@cs.cmu.edu> Added #include <iterator> for
//         gcc 3.1.
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 012901: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "cNNTP.hpp"

#include <iterator>

namespace Toolbox {

void NNTPClient::getResponse (NNTPResponse &r) {
  std::string line;
  
  readLine(line);  
  r.push_back(line);

  if (setRCode(line,r))
    do {
      line.erase();
      readLine(line);
      if (line[0]=='.' && line[1]!='.') break;
      else r.push_back(line);
    } while (true);
}

int NNTPClient::connect (const std::string &iNetAddr, NNTPResponse &r) {
  if (openRemote(iNetAddr,119)!=-1) {
    getResponse(r);
    return 0;
  }
  else return -1;
}

int NNTPClient::exec (const std::string &cmd, NNTPResponse &r) {
  if (writeLine(cmd+'\n')!=-1) {
    getResponse(r);
    return r.code;
  }
  
  return -1;
}

std::ostream &operator << (std::ostream &os, const NNTPResponse &r) {
  copy(r.begin(),r.end(),std::ostream_iterator<std::string>(os,"\n"));
  return os;
}

};
