// ======================================================================
// cNNTP.hpp - a simple NNTP client
// 
// 122502: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Fixed bug in NNTPClient::setRCode():
//         control reaches the end of a non-void function (duh).
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 012901: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CNNTP_HPP
#define CNNTP_HPP

#include "cStrSock.hpp"
#include "tokStr.hpp"
#include "misc.hpp"

#include <iostream>
#include <string>
#include <list>

#include <stdlib.h>

namespace Toolbox {

struct NNTPResponse: public std::list<std::string> {
  int code;
};

std::ostream &operator << (std::ostream &os, const NNTPResponse &r);

class NNTPClient: protected ClientStreamSocket {
protected:

  // According to RFC 0977: setRCode() returns true iff the response 
  // consists of a sequence of lines
  // TO-DO: might need to be extended for some NNTP extensions
  bool setRCode (const std::string &str, NNTPResponse &r) {
    TokenString tokStr;

    tokStr.set(str);
    r.code=atoi(tokStr[0].c_str());

    if ((r.code>=100 && r.code<=199) || r.code==215 ||
	(r.code>=220 && r.code<=222) || 
	(r.code>=230 && r.code<=231)) return true;
    else if (r.code==211) return (!isInteger(tokStr[1]));
    return false;
  }

  void getResponse (NNTPResponse &r);

public:

  // connect() and disconnect() return -1 iff an error occurs
  int connect (const std::string &iNetAddr, NNTPResponse &r);
  int disconnect () { return close(); }

  // exec() returns the NNTP response code, or -1 iff no NNTP response can
  // be obtained due to some other errors
  int exec (const std::string &cmd, NNTPResponse &r);
};

};

#endif
