// ======================================================================
// cStrSock.cpp - a client stream socket.
// 
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 010101: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "cStrSock.hpp"

#include <string>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

namespace Toolbox {

#define SOCKET_READ_CHUNK_SIZE 4095

int ClientStreamSocket::openRemote (const std::string &iNetAddr, unsigned short int port) {
  int result,len;

  sockfd=socket(AF_INET,SOCK_STREAM,0);
  remoteAddr.sin_family=AF_INET;
  remoteAddr.sin_addr.s_addr=inet_addr(iNetAddr.c_str());
  remoteAddr.sin_port=htons(port);
  len=sizeof(remoteAddr);

  result=connect(sockfd,(sockaddr*)&remoteAddr,len);

  #ifndef NDEBUG
  if (result!=-1) openFlag=true;
  #endif      

  return result;
}

int ClientStreamSocket::openLocal (const std::string &path) {
  int result,len;

  sockfd=socket(AF_UNIX,SOCK_STREAM,0);
  localAddr.sun_family=AF_UNIX;
  strcpy(localAddr.sun_path,path.c_str());
  len=sizeof(localAddr);

  result=connect(sockfd,(sockaddr*)&localAddr,len);

  #ifndef NDEBUG
  if (result!=-1) openFlag=true;
  #endif      

  return result;
}

int ClientStreamSocket::writeLine (const std::string &str) {
  assert(openFlag);

  int numLeft,numRead,n;

  for (numLeft=str.length(),numRead=0;numLeft>0;) {
    if ((n=write(sockfd,str.c_str()+numRead,numLeft))==-1) return -1;
    numRead+=n;
    numLeft-=n;
  }
  
  return 0;
}

void ClientStreamSocket::readLine (std::string &str) {
  assert(openFlag);

  static std::string nextStr;
  std::string::size_type i;

  if ((i=nextStr.find('\n'))!=std::string::npos) {
    str+=nextStr.substr(0,i);
    nextStr.erase(0,i+1);
    return;
  }

  char buf[SOCKET_READ_CHUNK_SIZE+1];
  int n;
  bool cont=true;

  do {
    str+=nextStr;
    
    n=read(sockfd,buf,SOCKET_READ_CHUNK_SIZE);
    buf[n]=0;
    nextStr=buf;
    
    if ((i=nextStr.find('\n'))!=std::string::npos) {
      str+=nextStr.substr(0,i);
      nextStr.erase(0,i+1);
      cont=false;
    }
  } while (cont);
}

};
