// ======================================================================
// cStrSock.hpp - a client stream socket.
// 
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 010101: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CSTRSOCK_HPP
#define CSTRSOCK_HPP

#include "tbconsts.hpp"

#include <string>

#include <assert.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <unistd.h>

namespace Toolbox {

class ClientStreamSocket {
  int sockfd;
  sockaddr_un localAddr;
  sockaddr_in remoteAddr;

#ifndef NDEBUG
  bool openFlag;
#endif

public:

  ClientStreamSocket () {
#ifndef NDEBUG
    openFlag=false;
#endif    
  }

  // openRemote(), openLocal() and close() returns -1 iff an error occurs

  int openRemote (const std::string &iNetAddr, unsigned short int port);
  int openLocal (const std::string &path);

  int close () {
    assert(openFlag);

    #ifndef NDEBUG
    openFlag=false;
    #endif

    return ::close(sockfd);
  }

  // writeLine() returns -1 iff an error occurs
  // readLine() *appends* 'str' with the chars read from the socket

  int writeLine (const std::string &str);
  void readLine (std::string &str);
};

};

#endif
