// ======================================================================
// cfgTst.hpp - testing both generation and analysis CFG classes.
// 
// 120602: Benjamin Han <benhdj@cs.cmu.edu> Created (total rewrite).
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "cfgGen.hpp"
#include "cfgAnalysis.hpp"

#include <iostream>

using namespace std;
using namespace Toolbox;

// printing an RHS
template <class RLiteral>
ostream &operator << (ostream &os, const RHS<RLiteral> &rhs) {
  typename RHS<RLiteral>::const_iterator ri;

  os<<*(ri=rhs.begin());
  while (++ri!=rhs.end()) os<<' '<<*ri;

  return os;
}

typedef CFGGen<string,string,double> PCFGGen;

#if _MSC_VER!=1200
typedef CFGAnalysis<string,string,double> PCFGAnalysis;
#endif

// ========== test the generation PCFG ==========
void testGen () {
  PCFGGen pcfgGen;
  PCFGGen::RHS rhsGen;
  PCFGGen::Iterator gi,giEnd;
  PCFGGen::RuleIterator ri,riFirst,riLast;

  cout<<"Testing generation grammar..."<<endl;

  rhsGen.clear();
  rhsGen.push_back("NP");
  rhsGen.push_back("VP");
  pcfgGen.addRule("S",rhsGen)=0.9;

  rhsGen.clear();
  rhsGen.push_back("NP");
  rhsGen.push_back("VP");
  rhsGen.push_back("PP");
  pcfgGen.addRule("S",rhsGen)=0.1;

  rhsGen.clear();
  rhsGen.push_back("N");
  pcfgGen.addRule("NP",rhsGen)=0.3;

  rhsGen.clear();
  rhsGen.push_back("DET");
  rhsGen.push_back("N");
  pcfgGen.addRule("NP",rhsGen)=0.7;

  rhsGen.clear();
  rhsGen.push_back("AUX");
  rhsGen.push_back("V");
  pcfgGen.addRule("VP",rhsGen)=0.3;

  rhsGen.clear();
  rhsGen.push_back("V");
  pcfgGen.addRule("VP",rhsGen)=0.2;

  rhsGen.clear();
  rhsGen.push_back("AUX");
  rhsGen.push_back("V");
  rhsGen.push_back("ADVP");
  pcfgGen.addRule("VP",rhsGen)=0.3;

  rhsGen.clear();
  rhsGen.push_back("V");
  rhsGen.push_back("ADVP");
  pcfgGen.addRule("VP",rhsGen)=0.2;

  cout<<"* # of rules = "<<pcfgGen.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgGen.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgGen.trieSize()<<endl;

  cout<<"* dump all the rules: "<<endl;
  giEnd=pcfgGen.end();
  for (gi=pcfgGen.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<gi.readLHS()<<" -> "<<ri->readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }

  cout<<"* find the rule with LHS \'S\':"<<endl;
  if (pcfgGen.findRule("S",riFirst,riLast))
    for (ri=riFirst;;++ri) {
      cout<<"S -> "<<ri->readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      

  cout<<"* find the rule with LHS \'NP\':"<<endl;
  if (pcfgGen.findRule("NP",riFirst,riLast))
    for (ri=riFirst;;++ri) {
      cout<<"NP -> "<<ri->readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }

#ifdef WITH_REMOVE_RULE
  cout<<"* remove rule \"VP -> V\"."<<endl;
  pcfgGen.findRule("VP",riFirst,riLast);
  ++(ri=riFirst);   // cuz we know the ordering, otherwise a search is needed
  pcfgGen.removeRule(ri);
  cout<<"* remove rule \"NP -> N\"."<<endl;
  pcfgGen.findRule("NP",riFirst,riLast);
  ri=riFirst;
  pcfgGen.removeRule(ri);
  cout<<"* remove rule \"S -> NP VP PP\"."<<endl;
  pcfgGen.findRule("S",riFirst,riLast);
  ri=riLast;
  pcfgGen.removeRule(ri);

  cout<<"* # of rules = "<<pcfgGen.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgGen.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgGen.trieSize()<<endl;

  cout<<"* dump all remaining rules:"<<endl;
  giEnd=pcfgGen.end();
  for (gi=pcfgGen.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<gi.readLHS()<<" -> "<<ri->readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }

  cout<<"* remove rule \"S -> NP VP\"."<<endl;
  pcfgGen.findRule("S",riFirst,riLast);
  ri=riFirst;
  pcfgGen.removeRule(ri);

  cout<<"* # of rules = "<<pcfgGen.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgGen.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgGen.trieSize()<<endl;

  cout<<"* dump all remaining rules:"<<endl;
  giEnd=pcfgGen.end();
  for (gi=pcfgGen.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<gi.readLHS()<<" -> "<<ri->readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }
#endif

  cout<<"* clear the grammar."<<endl;
  pcfgGen.clear();
  cout<<"* # of rules = "<<pcfgGen.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgGen.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgGen.trieSize()<<endl;
}

#if _MSC_VER!=1200
// ========== test the analysis PCFG ==========
void testAnalysis () {
  PCFGAnalysis pcfgAnalysis;
  PCFGAnalysis::RHS rhsAnalysis;
  PCFGAnalysis::Iterator gi,giEnd;
  PCFGAnalysis::RuleIterator ri,riFirst,riLast;

  cout<<endl<<"Testing analysis grammar..."<<endl;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("NP");
  rhsAnalysis.push_back("VP");
  pcfgAnalysis.addRule("S",rhsAnalysis)=0.9;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("NP");
  rhsAnalysis.push_back("VP");
  rhsAnalysis.push_back("PP");
  pcfgAnalysis.addRule("S",rhsAnalysis)=0.1;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("N");
  pcfgAnalysis.addRule("NP",rhsAnalysis)=0.3;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("DET");
  rhsAnalysis.push_back("N");
  pcfgAnalysis.addRule("NP",rhsAnalysis)=0.7;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("AUX");
  rhsAnalysis.push_back("V");
  pcfgAnalysis.addRule("VP",rhsAnalysis)=0.3;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("V");
  pcfgAnalysis.addRule("VP",rhsAnalysis)=0.2;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("AUX");
  rhsAnalysis.push_back("V");
  rhsAnalysis.push_back("ADVP");
  pcfgAnalysis.addRule("VP",rhsAnalysis)=0.3;

  rhsAnalysis.clear();
  rhsAnalysis.push_back("V");
  rhsAnalysis.push_back("ADVP");
  pcfgAnalysis.addRule("VP",rhsAnalysis)=0.2;

  cout<<"* # of rules = "<<pcfgAnalysis.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgAnalysis.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgAnalysis.trieSize()<<endl;

  cout<<"* dump all the rules: "<<endl;
  giEnd=pcfgAnalysis.end();
  for (gi=pcfgAnalysis.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<ri->readLHS()<<" -> "<<gi.readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }

  cout<<"* find the rule with RHS \"NP VP\":"<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("NP");
  rhsAnalysis.push_back("VP");
  if (pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast))
    for (ri=riFirst;;++ri) {
      cout<<ri->readLHS()<<" -> NP VP: "<<ri->body<<endl;
      if (ri==riLast) break;
    }      

  cout<<"* find the rule with RHS \"DET N\":"<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("DET");
  rhsAnalysis.push_back("N");
  if (pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast))
    for (ri=riFirst;;++ri) {
      cout<<ri->readLHS()<<" -> DET N: "<<ri->body<<endl;
      if (ri==riLast) break;
    }

#ifdef WITH_REMOVE_RULE
  cout<<"* remove rule \"VP -> V\"."<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("V");
  pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast);
  ri=riFirst;   // cuz we know the ordering, otherwise a search is needed
  pcfgAnalysis.removeRule(ri);
  cout<<"* remove rule \"NP -> N\"."<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("N");
  pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast);
  ri=riFirst;
  pcfgAnalysis.removeRule(ri);
  cout<<"* remove rule \"S -> NP VP PP\"."<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("NP");
  rhsAnalysis.push_back("VP");
  rhsAnalysis.push_back("PP");
  pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast);
  ri=riFirst;
  pcfgAnalysis.removeRule(ri);

  cout<<"* # of rules = "<<pcfgAnalysis.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgAnalysis.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgAnalysis.trieSize()<<endl;  

  cout<<"* dump all the rules: "<<endl;
  giEnd=pcfgAnalysis.end();
  for (gi=pcfgAnalysis.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<ri->readLHS()<<" -> "<<gi.readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }

  cout<<"* remove rule \"S -> NP VP\"."<<endl;
  rhsAnalysis.clear();
  rhsAnalysis.push_back("NP");
  rhsAnalysis.push_back("VP");
  pcfgAnalysis.findRule(rhsAnalysis,riFirst,riLast);
  ri=riFirst;
  pcfgAnalysis.removeRule(ri);

  cout<<"* # of rules = "<<pcfgAnalysis.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgAnalysis.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgAnalysis.trieSize()<<endl;  

  cout<<"* dump all the rules: "<<endl;
  giEnd=pcfgAnalysis.end();
  for (gi=pcfgAnalysis.begin();gi!=giEnd;++gi) {
    riFirst=gi.first();
    riLast=gi.last();
    for (ri=riFirst;;++ri) {
      cout<<ri->readLHS()<<" -> "<<gi.readRHS()<<": "<<ri->body<<endl;
      if (ri==riLast) break;
    }      
  }
#endif

  cout<<"* clear the grammar."<<endl;
  pcfgAnalysis.clear();
  cout<<"* # of rules = "<<pcfgAnalysis.size()<<endl;
  cout<<"* # of unique LHS literals = "<<pcfgAnalysis.lhsSize()<<endl;
  cout<<"* size of RHS trie = "<<pcfgAnalysis.trieSize()<<endl;  
}

#endif

int main () {
  testGen();

#if _MSC_VER!=1200
  testAnalysis();
#endif

  return 0;
}
