// ======================================================================
// except.hpp - simple exception handling
// 
// 121602: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Minor revisions to supress warnings
//         under -pedantic -Wall.
// 071801: Benjamin Han <benhdj@cs.cmu.edu> Minor revision.
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 101900: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef EXCEPT_HPP
#define EXCEPT_HPP

#include "tbconsts.hpp"

#include <iostream>
#include <string>

namespace Toolbox {

#define EX_PLACEHOLDER "%s"

// ===========================================================================
// Exception class
// ===========================================================================

class Exception {
#if _MSC_VER==1200
public:
#else
  friend std::ostream &operator<< (std::ostream &s, const Exception &tex);
#endif

  std::string msg;

public:

  // When print a Exception the EX_PLACEHOLDER in msg will be
  // replaced by arg.
  Exception (const std::string &msg, const std::string &arg) {
    std::string::size_type pos;
    Exception::msg=msg;

    if ((pos=Exception::msg.find(EX_PLACEHOLDER))!=std::string::npos) {
	Exception::msg.erase(pos,strlen(EX_PLACEHOLDER));
	Exception::msg.insert(pos,arg);
      }
  }
};

inline std::ostream &operator<< (std::ostream &s, const Exception &tex) {
  return s<<tex.msg;
}

// ===========================================================================
// Macro DEFINE_SIMPLE_EXCEPTION: Quickly define a specialized
// exception with customized error message.
// ===========================================================================

#define DEFINE_SIMPLE_EXCEPTION(exClassName,exMsg)\
class exClassName: public Exception {\
public:\
exClassName (const std::string &arg):Exception(exMsg,arg) {}\
}

};

#endif
