// ======================================================================
// gaTst.cpp - demonstrating the use of graph algorithms
// 
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Added randomized tests for
//         subgraph isomorphism tests and repeated subgraph isomorphism
//         tests.
// 083001: Benjamin Han <benhdj@cs.cmu.edu> Added randomized tests for
//         graph isomorphism tests.
// 081001: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//#define GA_DEBUG

#include "graphAlgorithm.hpp"
#include "random.hpp"

#include <list>
#include <vector>

using namespace Toolbox;
using namespace std;

/*
//big test
#define NUM_VDATA 6
#define NUM_V_PER_DATA 4
#define NUM_EDGE 288

#define NUM_MODEL_GRAPH 10
#define NUM_INPUT_GRAPH 20

#define NUM_EXTENDED_VERTEX 12
#define NUM_EXTENDED_EDGE 144
*/

// small test
#define NUM_VDATA 2
#define NUM_V_PER_DATA 2
#define NUM_EDGE 8

#define NUM_MODEL_GRAPH 10
#define NUM_INPUT_GRAPH 20

#define NUM_EXTENDED_VERTEX 2
#define NUM_EXTENDED_EDGE 4

struct Label {
  char c;

  Label () {}
  Label (char c):c(c) {}

  bool operator < (const Label &other) const { return c<other.c; }
  bool operator == (const Label &other) const { return c==other.c; }
  bool operator != (const Label &other) const { return c!=other.c; }
};

ostream &operator << (ostream &os, const Label &l) {
  return os<<l.c;
}

struct Double {
  double d;

  Double () {}
  Double (double d):d(d) {}

  bool operator == (const Double &other) const { return (d==other.d); }
};

ostream &operator << (ostream &os, const Double &d) {
  return os<<d.d;
}

typedef list<Graph<Label,Double>::Iterator> GIList;

void randomGraph (map<char,GIList> &vMap, Graph<Label,Double> &g) {
  GIList::iterator srcGLIter,destGLIter;
  char c;
  int i,j,k;

  // construct g randomly to have 4 different vertex labels, with each one 
  // having 3 vertices; 72 edges are added randomly (the # of edges in the 
  // fully connected g1, including the self-looping edges, is 144)
  vMap.clear();
  g.clear();
  for (i=0;i<NUM_VDATA;++i) {
    c='a'+i;
    for (j=0;j<NUM_V_PER_DATA;++j)
      vMap[c].push_back(g.add(c));
  }
  
  for (i=0;i<NUM_EDGE;++i) {
    GIList &srcGL=vMap[random()%NUM_VDATA+'a'];
    GIList &destGL=vMap[random()%NUM_VDATA+'a'];
    
    k=random()%NUM_V_PER_DATA+1;
    for (srcGLIter=srcGL.begin(),j=0;j<k;++srcGLIter,++j);
    if (k==NUM_V_PER_DATA) --srcGLIter;
    
    k=random()%NUM_V_PER_DATA+1;
    for (destGLIter=destGL.begin(),j=0;j<k;++destGLIter,++j);
    if (k==NUM_V_PER_DATA) --destGLIter;
    
    srcGLIter->addEdgeTo(*destGLIter,Double(random()%1000/1000.0));
  }
}

int main () {
  Graph<Label,Double> g1,g2;
  Graph<Label,Double>::Iterator gi1,gi2;
  map<char,GIList> vMap;
  GIsomorphism<Label,Double> isoFunc;
  SubGIsomorphism<Label,Double> sIsoFunc;
  GIsoMap<Graph<Label,Double> > giMap;
  GIsoMap<Graph<Label,Double> >::const_iterator gmi;
  char c;
  int i,j,k,l;
  
  // ==========================================================================
  // constructing graph g1 and g2
  for (c='a';c<='h';++c) vMap[c].push_back(g1.add(c));
  vMap['a'].begin()->addEdgeTo(*vMap['b'].begin(),1.0);
  vMap['b'].begin()->addEdgeTo(*vMap['c'].begin(),0.33);
  vMap['b'].begin()->addEdgeTo(*vMap['e'].begin(),0.33);
  vMap['b'].begin()->addEdgeTo(*vMap['f'].begin(),0.33);
  vMap['c'].begin()->addEdgeTo(*vMap['d'].begin(),0.5);
  vMap['c'].begin()->addEdgeTo(*vMap['g'].begin(),0.5);
  vMap['d'].begin()->addEdgeTo(*vMap['c'].begin(),0.5);
  vMap['d'].begin()->addEdgeTo(*vMap['h'].begin(),0.5);
  vMap['e'].begin()->addEdgeTo(*vMap['a'].begin(),0.5);
  vMap['e'].begin()->addEdgeTo(*vMap['f'].begin(),0.5);
  vMap['f'].begin()->addEdgeTo(*vMap['g'].begin(),1.0);
  vMap['g'].begin()->addEdgeTo(*vMap['f'].begin(),0.5);
  vMap['g'].begin()->addEdgeTo(*vMap['h'].begin(),0.5);
  vMap['h'].begin()->addEdgeTo(*vMap['h'].begin(),1.0);  // self-looping edge

  g2=g1;

  // ==========================================================================
  // show graph g1 and g2
  cout<<"* Graph g1:"<<endl<<g1<<endl;
  cout<<endl<<"* Graph g2:"<<endl<<g2<<endl;

  // ==========================================================================
  // Isomorphism test for g1 and g2
  cout<<endl<<"* Isomorphism test for g1 and g2:"<<endl;
  isoFunc.init(g1);
  cout<<(isoFunc(g2)?"yes":"no")<<endl;

  g2.find(Label('c')).getEdge(g2.find(Label('d')))=0.9;
  cout<<endl<<"* Isomorphism test for g1 and the modified g2:"<<endl;
  cout<<(isoFunc(g2)?"yes":"no")<<endl;

  g2=g1;
  g2.find(Label('c')).removeEdgeTo(g2.find(Label('d')));
  cout<<endl<<"* Isomorphism test for g1 and the modified g2:"<<endl;
  cout<<(isoFunc(g2)?"yes":"no")<<endl;

  g2=g1;
  g2.pruneOne(g2.find(Label('c')));
  cout<<endl<<"* Isomorphism test for g1 and the modified g2:"<<endl;
  cout<<(isoFunc(g2)?"yes":"no")<<endl;

  // ==========================================================================
  // Randomized graph isomorphism tests
  cout<<endl<<"======================================================================"<<endl;
  cout<<"Randomized isomorphism tests"<<endl;
  initRandom();

  for (i=0;i<NUM_MODEL_GRAPH;++i) {
    randomGraph(vMap,g1);
    isoFunc.init(g1);
    
    cout<<endl<<"* Isomorphism test for g1 and itself: ";
    if (isoFunc(g1)) cout<<"SUCC."<<endl;
    else {
      cout<<"FAIL: false."<<endl;
      cout<<"Graph g1:"<<endl<<g1<<endl;
    }
    
    g2=g1;
    cout<<endl<<"* Isomorphism test for g1 and g2 (assigned from g1): ";
    if (isoFunc(g2)) cout<<"SUCC."<<endl;
    else {
      cout<<"FAIL: false."<<endl;
      cout<<"Graph g1:"<<endl<<g1<<endl;
      cout<<"Graph g2:"<<endl<<g2<<endl;
    }
    
    cout<<endl<<"* Tests for "<<NUM_INPUT_GRAPH<<" randomly modified g2:"<<endl;
    for (j=0;j<NUM_INPUT_GRAPH;++j,g2=g1) {
      gi1=g2.find(Label(c=random()%NUM_VDATA+'a'));
      for (k=random()%vMap[c].size();k;--k,gi1=g2.findNext());
      
      gi2=g2.find(Label(c=random()%NUM_VDATA+'a'));
      for (k=random()%vMap[c].size();k;--k,gi2=g2.findNext());
      
      if (gi1.hasEdgeTo(gi2)) {
	cout<<"Edge removal: ";
	gi1.removeEdgeTo(gi2);
      }
      else {
	cout<<"Vertex removal: ";
	g2.pruneOne(gi1);
      }
      
      if (!isoFunc(g2)) cout<<"SUCC."<<endl;
      else {
	cout<<"FAIL: true."<<endl;
	cout<<"Graph g1:"<<endl<<g1<<endl;
	cout<<"Graph g2:"<<endl<<g2<<endl;
      }
    }
  }
  
  // ==========================================================================
  // Randomized subgraph isomorphism tests
  cout<<endl<<"======================================================================"<<endl;
  cout<<"Randomized subgraph isomorphism tests"<<endl;

  vector<Graph<Label,Double>::Iterator> newVertices(NUM_EXTENDED_VERTEX);

  for (i=0;i<NUM_MODEL_GRAPH;++i) {
    randomGraph(vMap,g1);
    sIsoFunc.init(g1);
    
    cout<<endl<<"* Tests for "<<NUM_INPUT_GRAPH<<" g2 randomly extended from g1:"<<endl;
    for (j=0,g2=g1;j<NUM_INPUT_GRAPH;++j,g2=g1) {
      // randomly add vertices
      for (k=0;k<NUM_EXTENDED_VERTEX;++k)
	newVertices[k]=g2.add(random()%NUM_VDATA+'a');
      
      // randomly add edges
      for (k=0;k<NUM_EXTENDED_EDGE;++k) {
	if (random()%2) {  // randomly pick a direction
	  gi1=newVertices[random()%NUM_EXTENDED_VERTEX];
	  gi2=g2.find(Label(c=random()%NUM_VDATA+'a'));
	  for (l=random()%vMap[c].size();l;--l,gi2=g2.findNext());
	}
	else {
	  gi1=g2.find(Label(c=random()%NUM_VDATA+'a'));
	  for (l=random()%vMap[c].size();l;--l,gi1=g2.findNext());
	  gi2=newVertices[random()%NUM_EXTENDED_VERTEX];
	}
	
	if (!gi1.hasEdgeTo(gi2))
	  gi1.addEdgeTo(gi2,Double(random()%1000/1000.0));
      }
      
      cout<<"Test result: ";
      if (sIsoFunc(g2)) cout<<"SUCC."<<endl;
      else {
	cout<<"FAIL: false."<<endl;
	cout<<"Graph g1:"<<endl<<g1<<endl;
	cout<<"Graph g2:"<<endl<<g2<<endl;
      }
    }
  }

  // ==========================================================================
  // Repeated subgraph isomorphism tests
  cout<<endl<<"======================================================================"<<endl;
  cout<<"Repeated subgraph isomorphism tests"<<endl;
  g2=g1;
  g2.copy(g1);
  cout<<"Graph g1:"<<endl<<g1<<endl;
  cout<<"Graph g2:"<<endl<<g2<<endl;
  sIsoFunc.init(g1);
  if (sIsoFunc(g2)) {
    i=0;
    do {
      cout<<endl<<"* Mapping "<<i++<<':'<<endl;
      
      // print out the mappings
      sIsoFunc.readMapping(giMap);
      for (gmi=giMap.begin();gmi!=giMap.end();++gmi) {
	cout<<*(gmi->first)<<'@';
	gmi->first.printVPtr(cout);
	cout<<" = "<<*(gmi->second)<<'@';
	gmi->second.printVPtr(cout);
	cout<<endl;
      }
    } while (sIsoFunc());
  }
  
  return 0;
}
