// ======================================================================
// indent.hpp - A parameterized indentation stream manipulator.
// 
// 121602: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 072102: Benjamin Han <benhdj@cs.cmu.edu> Added #include <iterator> for
//         gcc 3.1.
// 081601: Benjamin Han <benhdj@cs.cmu.edu> Removed const from the argument
//         of indent::readIndent().
// 052901: Benjamin Han <benhdj@cs.cmu.edu> Minor revision.
// 052301: Benjamin Han <benhdj@cs.cmu.edu> Added 'Note:' comments.
// 052301: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef INDENT_HPP
#define INDENT_HPP

#include "tbconsts.hpp"

#include <iterator>
#include <iostream>
#include <algorithm>

namespace Toolbox {

// NOTE: This is a sub-optimal way of implementing the indentation. *The*
// right way to do this is to define an IndentBuf deriving from streambuf
// and print the space characters there. The indent() will automatically
// install IndentBuf the first time it is called on a specific ostream.
// The reason I didn't do this for now is gcc doesn't provide a standard
// implementation on streambuf, traits_type, callback mechanisms of IO 
// streams.
// The biggest disadvantage of this code is it still requires a user to
// call indent::print(ostream&) to print out the leading spaces.

class indent {
  static const int index;
  int i;

  friend std::ostream &operator << (std::ostream &os, const indent &ind);

public:

  static int readIndent (std::ostream &os);
  static std::ostream &print (std::ostream &os);

  indent (int i):i(i) {}
};

// ============================ inline functions ============================

inline int indent::readIndent (std::ostream &os) {
  return os.iword(indent::index);
}

inline std::ostream &indent::print (std::ostream &os) {
  std::fill_n(std::ostream_iterator<char>(os),os.iword(index),' ');
  return os; 
}

inline std::ostream &operator << (std::ostream &os, const indent &ind) {
  os.iword(indent::index)+=ind.i;
  return os;
}

};

#endif
