// ======================================================================
// labelTrie.hpp - an unordered trie (prefix tree) structure with only 
//                 labels
// 
// 121602: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 112002: Benjamin Han <benhdj@cs.cmu.edu> nullTI -> nullTI().
// 111102: Benjamin Han <benhdj@cs.cmu.edu> Created by simplifying trie.hpp.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef LABELTRIE_HPP
#define LABELTRIE_HPP

#include "tree.hpp"

#include <list>

namespace Toolbox {

// ===========================================================================
// class LabelTrie: unordered trie (simplified from class Trie)
// ===========================================================================

template <class Label>
class LabelTrie: public UTree<Label> {
  
  Label key;

public:

  typedef std::list<Label> Prefix;

  // TO-DO: I'm not sure why I need to declare the following 2 typedef's;
  //        to illustrate in cfg.hpp, CFG<>::find(const RHS &...) I can use 
  //        RHSTrie::PreIterator with no problem, but if I didn't give the
  //        typedef's here, g++ 3.01 would've reported syntax error on
  //        Iterator addPrefix(...), and others.
  typedef typename UTree<Label>::Iterator Iterator;
  typedef typename UTree<Label>::DIterator DIterator;

  // insert a virtual node as the root
  LabelTrie () { insert(begin(),key); }

  Iterator addPrefix (const Prefix &prefix);
  Iterator findPrefix (const Prefix &prefix);

  // return true iff 'iter' has a daughter node with 'label'
  bool findNext (Iterator &iter, const Label &label);
};

template <class Label>
typename LabelTrie<Label>::Iterator
LabelTrie<Label>::addPrefix (const Prefix &prefix) {
  typename Prefix::const_iterator pi;
  Iterator iter;
  DIterator dIter;

  for (pi=prefix.begin(),iter=begin();pi!=prefix.end();++pi,iter=dIter) {
    key=*pi;
    if (!iter.find(key,dIter))
      // prefix partially exists - need to augment the trie
      do {
	iter=insert(iter.end(),key);
	if (++pi!=prefix.end()) key=*pi;
	  else return iter;
      } while (true);
  }
  return iter;
}

template <class Label>
typename LabelTrie<Label>::Iterator
LabelTrie<Label>::findPrefix (const Prefix &prefix) {
  typename Prefix::const_iterator pi;
  Iterator iter;
  DIterator dIter;

  for (pi=prefix.begin(),iter=begin();pi!=prefix.end();++pi,iter=dIter) {
    key=*pi;
    if (!iter.find(key,dIter)) return iter.nullTI();
  }
  return iter;
}

template <class Label>
bool LabelTrie<Label>::findNext (Iterator &iter, const Label &label) {
  DIterator dIter;

  key.label=label;
  if (iter.find(key,dIter)) {
    iter=dIter;
    return true;
  }
  else return false;
}

};

#endif
