// ======================================================================
// misc.cpp - misc. useful functions.
// 
// 061404: Benjamin Han <benhdj@cs.cmu.edu> isInteger(): make sure str is not
//         empty by testing errPtr != cStr; also typecast strtol() to int!
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Minor revisions to supress warnings
//         under -pedantic -Wall.
// 063001: Benjamin Han <benhdj@cs.cmu.edu> Added isInteger(const string&,int&).
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 010201: Benjamin Han <benhdj@cs.cmu.edu> Added isBlankLine() and
//         replace*() functions.
// 010101: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "misc.hpp"

#include <math.h>
#include <stdlib.h>


namespace Toolbox {

void intToString (int i, std::string &str) {
  int j,d,g;

  g=(int)log10(i);
  d=(int)pow(10,g);
  for (j=g,str.erase();j>0;j--) {
    str+=('0'+i/d);
    i%=d;
    d/=10;
  }
  str+=('0'+i);
}

bool isBlankLine (const std::string &str) {
  return (str.size()==0 || str.find_first_not_of(WHITESPACE_STR)==std::string::npos);
}

bool isInteger (const std::string &str) {
  const char *cStr=str.c_str();
  char *errPtr;
  
  strtol(cStr,&errPtr,10);
  return errPtr != cStr && *errPtr == '\0';
}

bool isInteger (const std::string &str, int &i) {
  const char *cStr=str.c_str();
  char *errPtr;
  
  i=(int)strtol(cStr,&errPtr,10);
  return errPtr != cStr && *errPtr == '\0';
}

bool replaceOnce (std::string &str, const std::string &pat, const std::string &nStr, std::string::size_type idx) {
  std::string::size_type i;
  
  if ((i=str.find(pat,idx))!=std::string::npos) {
    str.erase(i,pat.length());
    str.insert(i,nStr);
    return true;
  }
  else return false;
}

bool replaceAll (std::string &str, const std::string &pat, const std::string &nStr, std::string::size_type idx) {
  std::string::size_type i,s=nStr.length();

  // try to replace once first
  if ((i=str.find(pat,idx))!=std::string::npos) {
    str.erase(i,pat.length());
    str.insert(i,nStr);
  }
  else return false;
  
  for (i+=s;i<str.length() && (i=str.find(pat,i))!=std::string::npos;i+=s) {
    str.erase(i,pat.length());
    str.insert(i,nStr);      
  }
  
  return true;
}

};
