// ======================================================================
// regexTst.cpp - Demonstrating/testing regex.*.
// 
// 102901: Benjamin Han <benhdj@cs.cmu.edu> Added one more testing example.
// 071801: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "regex.hpp"

using namespace Toolbox;
using namespace std;

int main () {
  try {
    string literalPattern="(\\*\\+?|\\+)?(([^][*+:]+)?\\[([^][*+:]+:)?([^][*+:][^][*:]*)\\]|([^][*+:]+:)?([^][*+:][^][*:]*))(::[^][*+:]+)?";
    Regex r1(literalPattern);
    Regex r2(r1);
    Regex r3,r4;
    string str;
    int i,j;
    
    // using r1
    str="*[give-information+prescribe+medicine]";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r1.match(str)!=string::npos)
      for (i=0,j=r1.size();i<j;++i)
	if (r1.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r1.startPos(i),r1.length(i))<<'\"'<<endl;
    
    cout<<endl;
    
    // using r1 again
    str="*[time@identifiability=no]";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r1.match(str)!=string::npos)
      for (i=0,j=r1.size();i<j;++i)
	if (r1.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r1.startPos(i),r1.length(i))<<'\"'<<endl;
    
    cout<<endl;
    
    // using r2
    str="[give-information+prescribe+medicine]::MED";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r2.match(str)!=string::npos)
      for (i=0,j=r2.size();i<j;++i)
	if (r2.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r2.startPos(i),r2.length(i))<<'\"'<<endl;
    
    cout<<endl;
    
    // setting and using r3
    r3.set(literalPattern);
    str="[super_body-object-spec=]";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r3.match(str)!=string::npos)
      for (i=0,j=r3.size();i<j;++i)
	if (r3.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r3.startPos(i),r3.length(i))<<'\"'<<endl;
    
    cout<<endl;
    
    // setting and using r4
    r4=r3;
    str="[med:super_body-object-spec=]";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r4.match(str)!=string::npos)
      for (i=0,j=r4.size();i<j;++i)
	if (r4.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r4.startPos(i),r4.length(i))<<'\"'<<endl;
    
    cout<<endl;
    
    // using r4 again
    str="request-information+action";
    cout<<"Matching \""<<str<<"\":"<<endl;
    if (r4.match(str)!=string::npos)
      for (i=0,j=r4.size();i<j;++i)
	if (r4.isMatched(i))
	  cout<<"* "<<i<<": \""<<str.substr(r4.startPos(i),r4.length(i))<<'\"'<<endl;
  }
  catch (Exception &e) {
    cerr<<e<<endl;
  }
  
  return 0;
}
