// ======================================================================
// taTst.cpp - demonstrating the use of tree algorithms
// 
// 112002: Benjamin Han <benhdj@cs.cmu.edu> nullTI -> nullTI().
// 092601: Benjamin Han <benhdj@cs.cmu.edu> Minor revisions ("equivalen*"
//         to "isomorphi*").
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Minor revisions to supress
//         warnings under -pedantic -Wall; renamed Equivalence*<> to 
//         TIsomorphism*<>, and LexicographicalCompareRelaxed<> to
//         TLexicographicalCompareRelaxed<>.
// 080701: Benjamin Han <benhdj@cs.cmu.edu> Renamed all Homomorphic* 
//         identifiers to Equivalence*.
// 080201: Benjamin Han <benhdj@cs.cmu.edu> Renamed dBegin() and dEnd() to 
//         begin() and end() in _TreeIteratorBase<>.
// 072001: Benjamin Han <benhdj@cs.cmu.edu> Revised using new ways of
//         declaring tree iterators.
// 070401: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "treeAlgorithm.hpp"

using namespace Toolbox;
using namespace std;

typedef Tree<int> IntTree;

struct Op {
  bool operator () (Tree<int>::ConstIterator iter1, Tree<int>::ConstIterator iter2) const
  { return *iter1==*iter2; }
};

int main () {
  IntTree it1,it2;
  IntTree::Iterator iter1,iter2,iter3,iter4;
  TIsomorphism<IntTree> isoFunc;
  TIsomorphism<IntTree,Op> isoOpFunc;
  TIsomorphismRelaxed<IntTree> isoRelaxedFunc;
  TLexicographicalCompareRelaxed<IntTree> lexRelaxedFunc;
  
  cout<<"=============== Strict isomorphism tests ==============="<<endl;

  // (-1 (-2 0 1) 2 3) vs. itself
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  it2=it1;
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-2 (-2 0 1) 2 3)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-2);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),3);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-1 (-2 0 1))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1)) vs. (-1 (-2 0 1) 2 3)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),3);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-3 (-2 0 1) 2 3) vs. (-3 1 4 5 0)
  iter1=it1.insert(it1.begin(),-3);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-3);
  it2.insert(iter1.end(),1);
  it2.insert(iter1.end(),4);
  it2.insert(iter1.end(),5);
  it2.insert(iter1.end(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 0
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 1
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 1
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  cout<<endl<<"=============== Strict isomorphism tests with the homemade op ==============="<<endl;

  // (-1 (-2 0 1) 2 3) vs. itself
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  it2=it1;
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-2 (-2 0 1) 2 3)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-2);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),3);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-1 (-2 0 1))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1)) vs. (-1 (-2 0 1) 2 3)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),3);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-3 (-2 0 1) 2 3) vs. (-3 1 4 5 0)
  iter1=it1.insert(it1.begin(),-3);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-3);
  it2.insert(iter1.end(),1);
  it2.insert(iter1.end(),4);
  it2.insert(iter1.end(),5);
  it2.insert(iter1.end(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 0
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 1
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // 0 vs. 1
  it1.insert(it2.begin(),0);
  it2.insert(it2.begin(),1);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoOpFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  cout<<endl<<"=============== Relaxed isomorphism tests ==============="<<endl;

  // (-1 (-2 0 1) 2 3) vs. itself
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  it2=it1;
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-1 3 2 (-2 1 0))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),1);
  it2.insert(iter2.end(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 3 3) vs. (-1 3 2 (-2 1 0))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),3);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),1);
  it2.insert(iter2.end(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0) 3 2) vs. (-1 3 2 (-2 1 0))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter1.end(),3);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),1);
  it2.insert(iter2.end(),0);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (2 3)) vs. (-1 (0 (1 (2 3))))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),2);
  it1.insert(iter2.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),0);
  iter3=it2.insert(iter2.end(),1);
  iter4=it2.insert(iter3.end(),2);
  it2.insert(iter4.end(),3);
  cout<<it1<<" is isomorphic to "<<it2<<"? => "<<flush;
  cout<<(isoRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  cout<<endl<<"=============== Relaxed lexicographical comparison tests ==============="<<endl;

  // (-1 (-2 0 1) 2 3) vs. itself
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  it2=it1;
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 3) vs. (-1 (-2 0 1) 2)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it1.insert(iter1.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2) vs. (-1 (-2 0 1) 2 3)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),3);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 3 2) vs. (-1 (-2 0 1) 2)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),3);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2) vs. (-1 (-2 0 1) 3 2)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 3 2) vs. (-1 (-2 0 1) 2 4)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),3);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),4);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 4) vs. (-1 (-2 0 1) 3 2)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it2.insert(iter1.end(),4);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 3 2) vs. (-1 (-2 0 1) 2 4 5)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),3);
  it1.insert(iter1.end(),2);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),2);
  it2.insert(iter1.end(),4);
  it2.insert(iter1.end(),5);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (-2 0 1) 2 4 5) vs. (-1 (-2 0 1) 3 2)
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),-2);
  it1.insert(iter2.end(),0);
  it1.insert(iter2.end(),1);
  it1.insert(iter1.end(),2);
  it2.insert(iter1.end(),4);
  it2.insert(iter1.end(),5);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),-2);
  it2.insert(iter2.end(),0);
  it2.insert(iter2.end(),1);
  it2.insert(iter1.end(),3);
  it2.insert(iter1.end(),2);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (2 3)) vs. (-1 (0 (1 (2 3))))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),2);
  it1.insert(iter2.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),0);
  iter3=it2.insert(iter2.end(),1);
  iter4=it2.insert(iter3.end(),2);
  it2.insert(iter4.end(),3);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();

  // (-1 (0 (1 (2 3)))) vs. (-1 (2 3))
  iter1=it1.insert(it1.begin(),-1);
  iter2=it1.insert(iter1.end(),0);
  iter3=it1.insert(iter2.end(),1);
  iter4=it1.insert(iter3.end(),2);
  it1.insert(iter4.end(),3);
  iter1=it2.insert(it2.begin(),-1);
  iter2=it2.insert(iter1.end(),2);
  it2.insert(iter2.end(),3);
  cout<<it1<<" is less than "<<it2<<"? => "<<flush;
  cout<<(lexRelaxedFunc(it1.begin(),it2.begin())?"true":"false")<<endl;
  it1.clear();
  it2.clear();


  return 0;
}
