// ======================================================================
// tdTst.hpp - testing program for tokDict.hpp.
// 
// 110102: Benjamin Han <benhdj@cs.cmu.edu> No longer needed string conversion
//         in Data.
// 102702: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "vFile.hpp"
#include "tokDict.hpp"

#include <iostream>

#include <string>
#include <map>
#include <vector>

using namespace Toolbox;
using namespace std;

struct Data: public VFObject {
  string label;
  int i;

  Data () {}
  Data (const string &str, int i=0):label(str),i(i) {}

  bool operator < (const Data &other) const { return label<other.label; }

  void writeToVFile (VFile &vFile) const {
    vFile<<label;
    vFile<<i;
  }

  void readFromVFile (VFile &vFile) {
    vFile>>label;
    vFile>>i;
  }
};

ostream &operator << (ostream &os, const Data &d) {
  return os<<"label = "<<d.label<<"; i = "<<d.i;
}

int main () {
  TokenDict<Data> td;
  int idx,s;
  VFile f;

  cout<<"Add token \"apple\", id = "<<td.addToken(Data("apple"))<<endl;
  cout<<"Add token \"orange\", id = "<<td.addToken(Data("orange"))<<endl;
  cout<<"Add token \"banana\", id = "<<td.addToken(Data("banana"))<<endl;

  cout<<"Add token \"apple\", id = "<<td.addToken(Data("apple"))<<endl;

  cout<<"Read token 0: "<<td.readToken(0)<<endl;
  cout<<"Read token 1: "<<td.readToken(1)<<endl;

  cout<<"Update token \"apple\", id = "<<td.updateToken(Data("apple",10))<<endl;
  cout<<"Read token 0: "<<td.readToken(0)<<endl;

  cout<<"Update token \"strawberry\", id = "<<td.updateToken(Data("strawberry",20))<<endl;
  cout<<"Read token 3: "<<td.readToken(3)<<endl;

  cout<<"Add token \"peach\", result = "<<td.addToken(Data("peach"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  cout<<"Add token \"peach\", result = "<<td.addToken(Data("peach"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  cout<<"Update token \"peach\", result = "<<td.updateToken(Data("peach",30),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  td.setRollbackPoint();

  cout<<"Add token \"coke\", result = "<<td.addToken(Data("coke"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  cout<<"Add token \"coffee\", result = "<<td.addToken(Data("coffee"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  cout<<"Add token \"dew\", result = "<<td.addToken(Data("dew"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  td.rollback();

  cout<<"Add token \"grape\", result = "<<td.addToken(Data("grape"),idx)<<endl;
  cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  cout<<"Save to the file td.dat..."<<endl;
  f.open("td.dat",VFILE_WRITE);
  f<<td;
  f.close();

  td.clear();
  cout<<"Read from the file td.dat..."<<endl;
  f.open("td.dat",VFILE_READ);
  f>>td;
  for (idx=0,s=td.size();idx<s;++idx)
    cout<<"Read token "<<idx<<": "<<td.readToken(idx)<<endl;

  return 0;
}
