// ======================================================================
// treeTst.cpp - demonstrating the use of a Tree container and its various 
//               iterators
// 
// 112002: Benjamin Han <benhdj@cs.cmu.edu> nullTI -> nullTI().
// 111601: Benjamin Han <benhdj@cs.cmu.edu> Added tests for unordered trees.
// 110201: Benjamin Han <benhdj@cs.cmu.edu> Added tests for Tree<>::move().
// 090901: Benjamin Han <benhdj@cs.cmu.edu> Added mixed traversal tests, 
//         assignment tests and moveTo() tests for Tree<>::PathIterator.
// 080201: Benjamin Han <benhdj@cs.cmu.edu> Renamed dFindFirst() and
//         dFindNext() in _TreeIteratorBase<> to find() and findNext(),
//         respectively; renamed dBegin() and dEnd() to begin() and end() in
//         _TreeIteratorBase<>.
// 072001: Benjamin Han <benhdj@cs.cmu.edu> Revised using new ways of
//         declaring tree iterators.
// 062201: Benjamin Han <benhdj@cs.cmu.edu> Removed for_each test - no longer
//         valid since we need to call _TreeIteratorBase<>::update() to update
//         a node's data; also removed square() for for_each() tests.
// 061201: Benjamin Han <benhdj@cs.cmu.edu> Changed from "void main()" to
//         "int main ()".
// 042101: Benjamin Han <benhdj@cs.cmu.edu> Changed the license from GPL to
//         LGPL.
// 032801: Benjamin Han <benhdj@cs.cmu.edu> Added tests for dFindFirst();
//         renamed this file to "treeTst.cpp".
// 030201: Benjamin Han <benhdj@cs.cmu.edu> Added tests for PathIterator.
// 020201: Benjamin Han <benhdj@cs.cmu.edu> namespace Toolbox.
// 012501: Benjamin Han <benhdj@cs.cmu.edu> Created.
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "tree.hpp"

#include <iostream>
#include <algorithm>

using namespace Toolbox;
using namespace std;

struct Int {
  int i;

  Int () {}
  Int (int i):i(i) {}

  bool operator < (const Int &other) const { return i<other.i; }
  bool operator == (const Int &other) const { return i==other.i; }
  bool operator != (const Int &other) const { return i!=other.i; }
};

ostream &operator << (ostream &os, const Int &i) {
  return os<<i.i;
}

// testing ordered trees
void testTree () {
  Tree<Int> tree1,tree2;

  Tree<Int>::PreIterator ti1,ti11,ti12,ti13;
  Tree<Int>::ConstPostIterator ti2;
  Tree<Int>::ConstBFSIterator ti3;
  Tree<Int>::BFSIterator ti4;
  Tree<Int>::ConstFrontIterator ti5;

  Tree<Int>::DIterator tdi1;
  Tree<Int>::ConstDIterator tdi2;

  Tree<Int>::PathIterator::Path path;
  Tree<Int>::PathIterator pi1,pi2;

  int i;
  Int key;

  // ===========================================================================
  // constructing a tree

  ti1=tree1.insert(tree1.begin(),Int(0));

  ti11=tree1.insert(ti1.end(),Int(1));
  ti12=tree1.insert(ti1.end(),Int(2));
  ti13=tree1.insert(ti1.end(),Int(3));
  
  tree1.insert(ti11.end(),Int(4));
  ti11=tree1.insert(ti11.end(),Int(5));

  tree1.insert(ti11.end(),Int(6));
  tree1.insert(ti11.end(),Int(7));

  ti12=tree1.insert(ti12.end(),Int(8));
  tree1.insert(ti12.end(),Int(9));

  tree1.insert(ti13.end(),Int(10));
  tree1.insert(ti13.end(),Int(11));

  cout<<"* Tree constructed: "<<tree1.size()<<" nodes"<<endl;

  // ===========================================================================
  // iterator assignment test: should trigger compile-time errors appropriately

  tdi2=tdi1;        // OK: const=non-const
  //tdi1=tdi2;      // compile-time error: const can't be assigned to non-const
  ti3=ti4;          // OK: const=non-const
  //ti4=ti3;        // compile-time error: const can't be assigned to non-const
  
  tdi2=tree1.begin();  // OK: const=non-const
  tdi1=tree1.begin();  // OK: non-const=non-const

  tdi2=ti4;         // OK: const=non-const
  tdi2=ti3;         // OK: const=const
  tdi1=ti4;         // OK: non-const=non-const
  //tdi1=ti3;       // compile-time error: const can't be assigned to non-const

  ti3=tree1.begin();   // OK: const=non-const
  ti4=tree1.begin();   // OK: non-const=non-const

  ti3=tdi1;         // OK: const=non-const
  ti3=tdi2;         // OK: const=const
  ti4=tdi1;         // OK: non-const=non-const
  //ti4=tdi2;       // compile-time error: const can't be assigned to non-const

  ti1=ti4;          // OK: non-const=non-const (different agendas)
  ti2=ti3;          // OK: const=const (different agendas)
  ti2=ti1;          // OK: const=non-const (different agendas)
  //ti1=ti2;        // compile-time error: const can't be assigned to non-const

  // ===========================================================================  
  // tests for daughter iterator assignment with end()

  tdi1=tree1.begin().end();  // this points to a NULL node
  tdi1--;
  cout<<"* The root node of the right-most subtree through DIterator: "<<*tdi1<<endl;

  tdi2=tree1.begin().end();  // this points to a NULL node
  tdi2--;
  cout<<"* The root node of the right-most subtree through ConstDIterator: "<<*tdi2<<endl;

  // ===========================================================================  
  // tree traversal
  
  cout<<"* Preorder:"<<endl;
  for (ti1=tree1.begin();ti1!=ti1.nullTI();++ti1)
    cout<<*ti1<<endl;

  cout<<"* Postorder:"<<endl;
  for (ti2=tree1.begin();ti2!=ti2.nullTI();++ti2)
    cout<<*ti2<<endl;

  cout<<"* BFS:"<<endl;
  for (ti3=tree1.begin();ti3!=ti3.nullTI();++ti3)
    cout<<*ti3<<endl;

  cout<<"* Front:"<<endl;
  for (ti5=tree1.begin();ti5!=ti5.nullTI();++ti5)
    cout<<*ti5<<endl;

  cout<<"* Front of the leftmost subtree of the root:"<<endl;
  tdi2=tree1.begin().begin();
  for (ti5=tdi2;ti5!=ti5.nullTI();++ti5)
    cout<<*ti5<<endl;

  cout<<"* Traversal resumption by different iterators:"<<endl;
  ++(++(ti1=tree1.begin()));
  for (ti11=ti1;ti11!=ti11.nullTI();++ti11)
    cout<<*ti11<<endl;  

  cout<<"* Printing depths in preorder fashion:"<<endl;
  for (ti1=tree1.begin();ti1!=ti1.nullTI();++ti1)
    cout<<ti1.depth()<<endl;

  // ===========================================================================
  // daughter traversal

  cout<<"* Daughters of the root:"<<endl;  
  for (tdi1=tree1.begin().begin();tdi1!=tree1.begin().end();++tdi1)
    cout<<*tdi1<<endl;

  // ===========================================================================
  // Path traversal

  path.push_back(Int(2));
  path.push_back(Int(8));
  path.push_back(Int(9));

  pi1.setPath(path);

  cout<<"* Path traversal (path=[2,8,9]):"<<endl;
  for (pi1=tree1.begin();pi1!=pi1.nullTI();++pi1) cout<<*pi1<<endl;

  cout<<"* Path reverse traversal (path=[2,8,9]):"<<endl;
  for (--pi1;pi1!=tree1.begin();--pi1) cout<<*pi1<<endl;
  cout<<*pi1<<endl;

  cout<<"* Mixed path forward and reverse traversal test 1 (path=[2,8,9]):"<<endl;
  for (pi1=tree1.begin();*pi1!=Int(8);++pi1) cout<<*pi1<<endl;
  cout<<*pi1<<endl;
  --pi1;
  for (;pi1!=pi1.nullTI();++pi1) cout<<*pi1<<endl;

  cout<<"* Mixed path forward and reverse traversal test 2 (path=[2,8,9]):"<<endl;
  for (pi1=tree1.begin();pi1!=pi1.nullTI();++pi1) cout<<*pi1<<endl;
  --pi1;
  --pi1;
  --pi1;  // pi1 should point to 2
  cout<<"pi points to "<<*(pi1.readPI())<<endl;  // pi points to the next visiting node
  cout<<"current path dist = "<<pi1.pathDist()<<endl;
  for (;pi1!=pi1.nullTI();++pi1) cout<<*pi1<<endl;

  cout<<"* Path iterator assignment:"<<endl;
  pi1=tree1.begin();
  ++pi1;    // pi1 points to 2
  ++pi1;    // pi1 points to 8
  pi2=pi1;  // pi2 points to 8
  for (;pi2!=pi2.nullTI();++pi2) cout<<*pi2<<endl;

  cout<<"* Insertion via a path iterator:"<<endl;
  pi1=tree1.begin();
  ++pi1;
  ++pi1;   // point to 8

  tree1.insert(pi1,Int(12));

  ti1=tree1.begin();
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));

  cout<<"* Path iterator moveTo():"<<endl;
  path.clear();
  path.push_back(Int(2));
  path.push_back(Int(9));
  pi1.setPath(path);

  for (pi1=tree1.begin();*pi1!=Int(2);++pi1) cout<<*pi1<<endl;
  cout<<*pi1<<endl;
  tdi1=pi1.begin();
  ++tdi1;          // tdi1 points to 8
  pi1.moveTo(tdi1);
  cout<<"pi points to "<<*(pi1.readPI())<<endl;
  cout<<"current path dist = "<<pi1.pathDist()<<endl;
  cout<<*pi1<<endl;
  for (++pi1;pi1!=pi1.nullTI();++pi1) cout<<*pi1<<endl;
  cout<<"-- reverse --"<<endl;
  for (--pi1;pi1!=tree1.begin();--pi1) cout<<*pi1<<endl;
  cout<<*pi1<<endl;

  // ===========================================================================
  // find()

  cout<<"* Test of find():"<<endl;
  ti1=tree1.begin();
  if (ti1.find(Int(2),tdi1)) cout<<"true: "<<*tdi1<<endl;
  else cout<<"false"<<endl;

  // ===========================================================================
  // modifying nodes

  for (ti1=tree1.begin(),i=0;ti1!=ti1.nullTI();++ti1,++i) ti1.update(i);
  cout<<"* After modification; Preorder:"<<endl;
  for (ti1=tree1.begin();ti1!=ti1.nullTI();++ti1)  // show the modifications
    cout<<*ti1<<endl;
  
  // ===========================================================================
  // upward traversal via TreeIterator

  cout<<"* Upward traversal via TreeIterator:"<<endl;
  for (ti2=tree1.begin();ti2!=ti2.nullTI();ti2.up())
    cout<<*ti2<<endl;

  // ===========================================================================
  // upward traversal via TreeConstDIterator

  cout<<"* Upward traversal via TreeConstDIterator:"<<endl;
  ti2=tree1.begin();
  for (tdi2=ti2.up().begin();tdi2!=ti2.nullTI();tdi2.up())
    cout<<*tdi2<<endl;

  // ===========================================================================
  // tree copying and clearing

  tree2=tree1;
  tree1.clear();
  cout<<"* Tree copied with "<<tree2.size()<<" nodes; Preorder:"<<endl;
  for (ti1=tree2.begin();ti1!=ti1.nullTI();++ti1)  // show the modifications
    cout<<*ti1<<endl;

  cout<<"* Verify the depths are correct:"<<endl;
  for (ti1=tree2.begin();ti1!=ti1.nullTI();++ti1)
    cout<<ti1.depth()<<endl;  

  // ===========================================================================  
  // Printing daughters using copy()

  cout<<"* Printout of the daughters of the root:"<<endl; 
  copy(tree2.begin().begin(),tree2.begin().end(),ostream_iterator<Int>(cout,"\n"));

  // ===========================================================================
  // Printing an entire tree using copy()

  cout<<"* Printout of the whole tree:"<<endl;
  ti1=tree2.begin();
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));

  // ===========================================================================
  // linear search using find()
  // you can use bin_search() if the nodes are sorted in the order of
  // traversal for better performance
  
  key=16;
  cout<<"* Find "<<key<<":"<<endl;
  if (find(ti1,ti1.nullTI(),key)!=ti1.nullTI()) cout<<"Found!"<<endl;
  else cout<<"Not found!"<<endl;

  key=3;
  cout<<"* Find "<<key<<":"<<endl;
  if (find(ti1,ti1.nullTI(),key)!=ti1.nullTI()) cout<<"Found!"<<endl;
  else cout<<"Not found!"<<endl;

  // ===========================================================================
  // reporting distance
  
  cout<<"* Distance from the root and its first daughter:"<<endl;
  ++(ti11=ti1);
  cout<<distance(ti1,ti11)<<endl;

  cout<<"* Distance from the root and the last node in a complete traversal:"<<endl;
  cout<<distance(ti1,ti1.nullTI())<<endl;

  cout<<"* The size of the leftmost subtree:"<<endl;
  ++(ti11=ti1);
  ti11.start();
  cout<<distance(ti11,ti11.nullTI())<<endl;

  // ===========================================================================
  // pruning a tree (removing a subtree) using TreeIterator

  cout<<"* Pruning away a subtree using erase():"<<endl;
  tree1=tree2;      // keep a copy
  ++(ti11=ti1);     // remember ti1 is pointing to the root of tree2
  ti11=tree2.erase(ti11);
  ti1=tree2.begin();    // 're-synch' is necessary because some nodes no longer exist
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;
  cout<<"next position: "<<*ti11<<endl;
  cout<<"still next position: "<<*(++ti11)<<endl;

  cout<<"* Pruning away a subtree using prune():"<<endl;
  tree2=tree1;
  ti1=tree2.begin();    // 're-synch' is necessary because of the assignment
  ++(ti11=ti1);
  tree2.prune(ti11);
  ti1=tree2.begin();    // 're-synch' is necessary because some nodes no longer exist  
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;

  // ===========================================================================
  // pruning a tree (removing a subtree) using TreeDIterator

  cout<<"* Pruning away the middle subtree using prune():"<<endl;
  tree2=tree1;
  ++(tdi1=tree2.begin().begin());  // prune the middle tree
  tree2.prune(tdi1);
  ti1=tree2.begin();    // 're-synch' is necessary because some nodes no longer exist  
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;

  cout<<"* Pruning away the middle subtree using erase():"<<endl;
  tree2=tree1;
  ++(tdi1=tree2.begin().begin());  // prune the middle tree
  tdi1=tree2.erase(tdi1);
  ti1=tree2.begin();    // 're-synch' is necessary because some nodes no longer exist
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;
  cout<<"next position: "<<*tdi1<<endl;

  // ===========================================================================
  // insert a subtree into a tree

  cout<<"* Restoring the middle subtree by re-inserting it into the tree"<<endl;
  ++(tdi2=tree1.begin().begin());  // tdi2 points to the middle subtree
  tree2.insert(tdi1,tdi2);
  ti1=tree2.begin();
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;

  cout<<"* Verify the depths are correct:"<<endl;
  for (ti1=tree2.begin();ti1!=ti1.nullTI();++ti1)
    cout<<ti1.depth()<<endl;  

  // ===========================================================================
  // Move a subtree to another position (not in the same lineage) within a tree
  cout<<"* move the right-most subtree of the root before the node 7"<<endl;
  --(tdi1=tree2.begin().end());   // tdi1 points to the right most subtree of the root
  ++(tdi2=tree2.begin().begin());
  tdi2=tdi2.begin();              // tdi2 points to the node 7
  tree2.move(tdi1,tdi2);

  ti1=tree2.begin();
  copy(ti1,ti1.nullTI(),ostream_iterator<Int>(cout,"\n"));
  cout<<tree2.size()<<" nodes"<<endl;
}

// testing unordered trees
void testUTree () {
  UTree<Int> tree1,tree2;

  UTree<Int>::PreIterator ti1,ti11,ti12,ti13;

  // ===========================================================================
  // constructing a tree

  ti1=tree1.insert(tree1.begin(),Int(0));

  ti11=tree1.insert(ti1.end(),Int(1));
  ti12=tree1.insert(ti1.end(),Int(2));
  ti13=tree1.insert(ti1.end(),Int(3));
  
  tree1.insert(ti11.end(),Int(4));
  ti11=tree1.insert(ti11.end(),Int(5));

  tree1.insert(ti11.end(),Int(6));
  tree1.insert(ti11.end(),Int(7));

  ti12=tree1.insert(ti12.end(),Int(8));
  tree1.insert(ti12.end(),Int(9));

  tree1.insert(ti13.end(),Int(10));
  tree1.insert(ti13.end(),Int(11));

  cout<<"* Tree constructed: "<<tree1.size()<<" nodes"<<endl;  

  cout<<"* Preorder:"<<endl;
  for (ti1=tree1.begin();ti1!=ti1.nullTI();++ti1)
    cout<<*ti1<<endl;

  tree2=tree1;
  cout<<"* Preorder:"<<endl;
  for (ti1=tree2.begin();ti1!=ti1.nullTI();++ti1)
    cout<<*ti1<<endl;
}

int main (void) {

  cout<<"======================================================================"<<endl;
  cout<<"Testing ordered trees"<<endl;
  cout<<"======================================================================"<<endl;
  testTree();

  cout<<endl;
  cout<<"======================================================================"<<endl;
  cout<<"Testing unordered trees"<<endl;
  cout<<"======================================================================"<<endl;
  testUTree();

  return 0;
}
