// ======================================================================
// vFile.cpp - Versatile files: kitchen sink for data storage.
// 
// 121602: Benjamin Han <benhdj@cs.cmu.edu> removed "using namespace std;" 
//         and inserted std:: prefix where needed.
// 120202: Benjamin Han <benhdj@cs.cmu.edu> Created (total rewrite).
// ======================================================================

//    Copyright (C) 2000-2004 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "vFile.hpp"

namespace Toolbox {
  
void VFile::open (const std::string &fileName, int mode) {
  if (fsPtr) close();
  
  switch (mode) {
  case VFILE_READ:
    
    fsPtr=new std::fstream;
    fsPtr->open(fileName.c_str(),std::ios::in);
    if (!(*fsPtr)) {
      fsPtr->close();
      delete fsPtr;
      fsPtr=NULL;
      throw VFileOpenError(fileName+" for reading");
    }
    break;
    
  case VFILE_WRITE:
    fsPtr=new std::fstream;
    fsPtr->open(fileName.c_str(),std::ios::out);
    if (!(*fsPtr)) {
      fsPtr->close();
      delete fsPtr;
      fsPtr=NULL;
      throw VFileOpenError(fileName+" for writing");
    }
    break;
  }
  
  VFile::mode=mode;
}
  
void VFile::close () {
  if (fsPtr) {
    fsPtr->close();
    delete fsPtr;
    fsPtr=NULL;
  }
  
  mode=VFILE_CLOSED;
}

};
